@@ -25,8 +25,15 @@ public:
 
 	engines::engine::status errorCode( const QString& e,int s ) const override ;
 
+	const QProcessEnvironment& getProcessEnvironment() const override ;
+
 	engines::engine::args command( const QByteArray& password,
 				       const engines::engine::cmdArgsList& args ) const override ;
 
-	void GUICreateOptionsinstance( QWidget * parent,engines::engine::function ) const override ;
+	void GUICreateOptions( const engines::engine::createGUIOptions& ) const override ;
+
+	void GUIMountOptions( const engines::engine::mountGUIOptions& ) const override ;
+private:
+	mutable QProcessEnvironment m_environment ;
+	const engines::versionGreaterOrEqual m_versionGreatorOrEqual_1_9_5 ;
 } ;
