@@ -20,6 +20,7 @@
 #include "gocryptfs.h"
 
 #include "gocryptfscreateoptions.h"
+#include "options.h"
 
 static engines::engine::BaseOptions _setOptions()
 {
@@ -80,8 +81,10 @@ static bool _set_if_found( const Function& function )
 	return false ;
 }
 
-void gocryptfs::updateOptions( engines::engine::options& opt ) const
+void gocryptfs::updateOptions( engines::engine::cmdArgsList::options& opt,bool creating ) const
 {
+	Q_UNUSED( creating )
+
 	opt.boolOptions.unlockInReverseMode = [ & ](){
 
 		if( opt.configFilePath.isEmpty() ){
@@ -175,7 +178,29 @@ engines::engine::status gocryptfs::errorCode( const QString& e,int s ) const
 	return engines::engine::status::backendFail ;
 }
 
-void gocryptfs::GUICreateOptionsinstance( QWidget * parent,engines::engine::function function ) const
+void gocryptfs::GUICreateOptions( const engines::engine::createGUIOptions& s ) const
 {
-	gocryptfscreateoptions::instance( parent,std::move( function ) ) ;
+	gocryptfscreateoptions::instance( *this,s ) ;
+}
+
+void gocryptfs::GUIMountOptions( const engines::engine::mountGUIOptions& s ) const
+{
+	auto& e = options::instance( *this,s ) ;
+
+	auto& ee = e.GUIOptions() ;
+
+	ee.enableKeyFile = false ;
+
+	ee.checkBoxChecked = s.mOpts.opts.unlockInReverseMode ;
+
+	ee.updateOptions = []( const ::options::Options& s ){
+
+		engines::engine::booleanOptions e ;
+
+		e.unlockInReverseMode = s.checkBoxChecked ;
+
+		return e ;
+	} ;
+
+	e.ShowUI() ;
 }
