@@ -27,7 +27,7 @@
 #include <QFile>
 #include <QCryptographicHash>
 
-static utility::result< QString > _config_path()
+static utility::qstring_result _config_path()
 {
 	QString m = settings::instance().ConfigLocation() + "/favorites/" ;
 
@@ -130,7 +130,7 @@ static void _log_error( const QString& msg,const QString& path )
 	utility::debug::showDebugWindow( msg + a ) ;
 }
 
-static void _add_entries( std::vector< favorites::entry >& e,const QString& path )
+utility2::result< favorites::entry > favorites::readFavoriteByPath( const QString& path ) const
 {
 	try {
 		SirikaliJson json( path,SirikaliJson::type::PATH ) ;
@@ -149,10 +149,17 @@ static void _add_entries( std::vector< favorites::entry >& e,const QString& path
 		m.reverseMode          = json.getBool( "reverseMode" ) ;
 		m.volumeNeedNoPassword = json.getBool( "volumeNeedNoPassword" ) ;
 
+		try {
+			m.keyFile      = json.getString( "keyFilePath" ) ;
+
+		}catch( ... ) {
+
+		}
+
 		favorites::triState::readTriState( json,m.readOnlyMode,"mountReadOnly" ) ;
 		favorites::triState::readTriState( json,m.autoMount,"autoMountVolume" ) ;
 
-		e.emplace_back( std::move( m ) ) ;
+		return m ;
 
 	}catch( const SirikaliJson::exception& e ){
 
@@ -166,6 +173,8 @@ static void _add_entries( std::vector< favorites::entry >& e,const QString& path
 
 		_log_error( "Unknown error has occured",path ) ;
 	}
+
+	return {} ;
 }
 
 std::vector<favorites::entry> favorites::readFavorites() const
@@ -181,17 +190,22 @@ std::vector<favorites::entry> favorites::readFavorites() const
 
 	const auto s = QDir( a ).entryList( QDir::Filter::Files | QDir::Filter::Hidden ) ;
 
-	std::vector<favorites::entry> e ;
+	std::vector< favorites::entry > e ;
 
 	for( const auto& it : s ){
 
-		_add_entries( e,a + it ) ;
+		auto mm = this->readFavoriteByPath( a + it ) ;
+
+		if( mm ){
+
+			e.emplace_back( mm.RValue() ) ;
+		}
 	}
 
 	return e ;
 }
 
-utility::result< favorites::entry > favorites::readFavorite( const QString& e,const QString& s ) const
+utility2::result< favorites::entry > favorites::readFavorite( const QString& e,const QString& s ) const
 {
 	if( s.isEmpty() ){
 
@@ -249,6 +263,7 @@ favorites::error favorites::add( const favorites::entry& e )
 		json[ "volumePath" ]           = e.volumePath ;
 		json[ "mountPointPath" ]       = e.mountPointPath ;
 		json[ "configFilePath" ]       = e.configFilePath ;
+		json[ "keyFilePath" ]          = e.keyFile ;
 		json[ "idleTimeOut" ]          = e.idleTimeOut ;
 		json[ "mountOptions" ]         = e.mountOptions ;
 		json[ "preMountCommand" ]      = e.preMountCommand ;
