@@ -55,7 +55,6 @@
 #include "siritask.h"
 #include "checkforupdates.h"
 #include "favorites.h"
-#include "walletconfig.h"
 #include "plugins.h"
 #include "crypto.h"
 #include "help.h"
@@ -138,6 +137,8 @@ configOptions::functions sirikali::configOption()
 
 void sirikali::closeApplication( int s,const QString& e )
 {
+	m_secrets.close() ;
+
 	if( utility::platformIsWindows() && m_ui && !m_emergencyShuttingDown ){
 
 		if( m_ui->tableWidget->rowCount() > 0 ){
@@ -153,7 +154,7 @@ void sirikali::closeApplication( int s,const QString& e )
 
 	if( !e.isEmpty() ){
 
-		utility::debug() << e ;
+		utility::debug::cerr() << e ;
 	}
 
 	if( m_ui ){
@@ -317,7 +318,7 @@ void sirikali::setUpApp( const QString& volume )
 
 		this->enableAll() ;
 	}else{
-		this->showMoungDialog( volume ) ;
+		this->autoMount( volume ) ;
 	}
 
 	this->startGUI( m ) ;
@@ -503,26 +504,6 @@ void sirikali::autoUpdateCheck()
 	m_checkUpdates.run( true ) ;
 }
 
-static void _connect_to_ssh_server( QWidget * parent,const favorites::entry& entry )
-{
-	auto s = siritask::encryptedFolderMount( entry ) ;
-
-	if( s == engines::engine::status::sshfsNotFound ){
-
-		auto e = QObject::tr( "Failed To Complete The Request.\nSshfs Executable Could Not Be Found." ) ;
-
-		DialogMsg( parent ).ShowUIOK( QObject::tr( "ERROR" ),e ) ;
-
-	}else if( s != engines::engine::status::success ){
-
-		auto e = QObject::tr( "Failed To Connect To \"%1\" And Below Log was Generated By The Backend.\n" ).arg( entry.volumePath ) ;
-
-		e += "\n----------------------------------------\n" + s.toMiniString() ;
-
-		DialogMsg( parent ).ShowUIOK( QObject::tr( "ERROR" ),e ) ;
-	}
-}
-
 void sirikali::favoriteClicked( QAction * ac )
 {
 	auto e = ac->objectName() ;
@@ -531,23 +512,44 @@ void sirikali::favoriteClicked( QAction * ac )
 
 		this->disableAll() ;
 
-		favorites2::instance( this,favorites::type::others,[ this ](){
+		favorites2::instance( this,m_secrets,[ this ](){
 
 			this->enableAll() ;
 
 			this->updateFavoritesInContextMenu() ;
 		} ) ;
-	}else{
+	}else{		
 		if( e == "Mount All" ){
 
-			this->mountMultipleVolumes( _readFavorites() ) ;
+			auto v = _readFavorites() ;
+
+			auto e = settings::instance().autoMountBackEnd() ;
+
+			if( e.isValid() ){
+
+				auto m = m_secrets.walletBk( e.bk() ) ;
+
+				m.open( [ & ](){
+
+					for( auto& it : v ){
+
+						auto s = m->readValue( it.first.volumePath ) ;
+
+						if( !s.isEmpty() ){
+
+							it.second = s ;
+						}
+					}
+
+					return true ;
+				} ) ;
+			}
+
+			this->mountMultipleVolumes( std::move( v ) ) ;
 		}else{
-			QString volumePath ;
-			QString mountPointPath ;
-
-			utility2::stringListToStrings( utility::split( e ),volumePath,mountPointPath ) ;
-
-			auto _found = [ & ]( const std::pair< favorites::entry,QByteArray >& e ){
+			auto _found = [ & ]( const std::pair< favorites::entry,QByteArray >& e,
+					     const QString& mountPointPath,
+					     const QString& volumePath ){
 
 				const auto& s = e.first ;
 
@@ -559,25 +561,16 @@ void sirikali::favoriteClicked( QAction * ac )
 				}
 			} ;
 
+			QString volumePath ;
+			QString mountPointPath ;
+
+			utility2::stringListToStrings( utility::split( e ),volumePath,mountPointPath ) ;
+
 			for( auto&& it : _readFavorites() ){
 
-				if( _found( it ) ){
-
-					siritask::mount m{ it.first.volumePath,QString(),siritask::Engine() } ;
-
-					auto s = siritask::mountEngine( m )->name() ;
-
-					if( it.first.volumeNeedNoPassword && s == "sshfs" ){
-
-						this->disableAll() ;
-
-						_connect_to_ssh_server( this,it.first ) ;
-
-						this->enableAll() ;
-					}else{
-						this->mountMultipleVolumes( this->autoUnlockVolumes( { std::move( it ) },true ) ) ;
-					}
+				if( _found( it,mountPointPath,volumePath ) ){
 
+					this->mountMultipleVolumes( this->autoUnlockVolumes( { std::move( it ) },true ) ) ;
 					break ;
 				}
 			}
@@ -608,6 +601,8 @@ void sirikali::startGUI( const std::vector< volumeInfo >& m )
 
 		this->autoUnlockVolumes( m ) ;
 	}
+
+	utility::applicationStarted() ;
 }
 
 void sirikali::createbackendwindow()
@@ -621,7 +616,7 @@ void sirikali::raiseWindow( const QString& volume )
 	this->raise() ;
 	this->show() ;
 	this->setWindowState( Qt::WindowActive ) ;
-	this->showMoungDialog( volume ) ;
+	this->autoMount( volume ) ;
 }
 
 int sirikali::start( QApplication& e )
@@ -802,15 +797,18 @@ void sirikali::unlockVolume( const QStringList& l )
 
 			m_mountInfo.announceEvents( false ) ;
 
-			engines::engine::options s( volume,
-						    m,key,
-						    idleTime,
-						    cPath,
-						    { mode,reverse,false,false },
-						    mOpt,
-						    QString() ) ;
+			engines::engine::booleanOptions mmm ;
 
-			auto e = siritask::encryptedFolderMount( s ) ;
+			mmm.unlockInReverseMode = reverse ;
+			mmm.unlockInReadOnly    = mode ;
+
+			engines::engine::mountGUIOptions::mountOptions mm( idleTime,
+									   cPath,
+									   mOpt,
+									   QString(),
+									   mmm ) ;
+
+			auto e = siritask::encryptedFolderMount( { volume,m,key,mm } ) ;
 
 			if( e == engines::engine::status::success ){
 
@@ -852,39 +850,35 @@ void sirikali::unlockVolume( const QStringList& l )
 
 		auto w = [ & ](){
 
-			namespace wxt = LXQt::Wallet ;
+			using wxt = LXQt::Wallet::BackEnd ;
 
-			auto _supported = [ & ]( wxt::BackEnd e,const char * s ){
+			auto _supported = [ & ]( wxt e,const char * s ){
 
-				return backEnd == s && wxt::backEndIsSupported( e ) ;
+				return backEnd == s && LXQt::Wallet::backEndIsSupported( e ) ;
 			} ;
 
-			if( _supported( wxt::BackEnd::internal,"internal" ) ){
+			if( _supported( wxt::internal,"internal" ) ){
 
-				auto s = m_secrets.walletBk( wxt::BackEnd::internal ) ;
+				return m_secrets.walletBk( wxt::internal ).getKey( volume ) ;
 
-				return utility::getKey( volume,s.bk() ) ;
+			}else if( _supported( wxt::libsecret,"gnomewallet" ) ||
+				  _supported( wxt::libsecret,"libsecret" ) ){
 
-			}else if( _supported( wxt::BackEnd::libsecret,"gnomewallet" ) ||
-				  _supported( wxt::BackEnd::libsecret,"libsecret" ) ){
+				return m_secrets.walletBk( wxt::libsecret ).getKey( volume ) ;
 
-				auto s = m_secrets.walletBk( wxt::BackEnd::libsecret ) ;
+			}else if( _supported( wxt::kwallet,"kwallet" ) ){
 
-				return utility::getKey( volume,s.bk() ) ;
+				return m_secrets.walletBk( wxt::kwallet ).getKey( volume ) ;
 
-			}else if( _supported( wxt::BackEnd::kwallet,"kwallet" ) ){
+			}else if( _supported( wxt::osxkeychain,"osxkeychain" ) ){
 
-				auto s = m_secrets.walletBk( wxt::BackEnd::kwallet ) ;
+				return m_secrets.walletBk( wxt::osxkeychain ).getKey( volume ) ;
 
-				return utility::getKey( volume,s.bk() ) ;
+			}else if( _supported( SiriKali::Windows::windowsWalletBackend(),"windows_dpapi" ) ){
 
-			}else if( _supported( wxt::BackEnd::osxkeychain,"osxkeychain" ) ){
-
-				auto s = m_secrets.walletBk( wxt::BackEnd::osxkeychain ) ;
-
-				return utility::getKey( volume,s.bk() ) ;
+				return m_secrets.walletBk( SiriKali::Windows::windowsWalletBackend() ).getKey( volume ) ;
 			}else{
-				return utility::wallet{ false,true,"" } ;
+				return secrets::wallet::walletKey{ false,true,"" } ;
 			}
 		}() ;
 
@@ -989,7 +983,7 @@ favorites::volumeList sirikali::autoUnlockVolumes( favorites::volumeList l,bool
 		return l ;
 	}
 
-	auto _mountVolumes = [ & ](){
+	auto s = m.open( [ & ](){
 
 		favorites::volumeList e ;
 
@@ -1001,6 +995,8 @@ favorites::volumeList sirikali::autoUnlockVolumes( favorites::volumeList l,bool
 
 				q.emplace_back( e.first,key ) ;
 			}else{
+				this->disableAll() ;
+
 				auto s = siritask::encryptedFolderMount( { e.first,key } ) ;
 
 				if( s == engines::engine::status::success ){
@@ -1014,7 +1010,13 @@ favorites::volumeList sirikali::autoUnlockVolumes( favorites::volumeList l,bool
 
 						this->updateList() ;
 					}
+				}else{
+					q.emplace_back( e.first,key ) ;
+
+					utility::debug() << "Automounting has failed because: " + s.toString() ;
 				}
+
+				this->enableAll() ;
 			}
 		} ;
 
@@ -1033,23 +1035,13 @@ favorites::volumeList sirikali::autoUnlockVolumes( favorites::volumeList l,bool
 		}
 
 		return e ;
-	} ;
+	} ) ;
 
+	if( s.has_value() ){
 
-	if( m->opened() ){
-
-		return _mountVolumes() ;
+		return s.RValue() ;
 	}else{
-		m->setImage( QIcon( ":/sirikali" ) ) ;
-
-		auto& s = settings::instance() ;
-
-		if( m->open( s.walletName( m->backEnd() ),s.applicationName() ) ){
-
-			return _mountVolumes() ;
-		}else{
-			return l ;
-		}
+		return l ;
 	}
 }
 
@@ -1185,15 +1177,22 @@ void sirikali::addToFavorites()
 {
 	auto table = m_ui->tableWidget ;
 
-	auto cp = tablewidget::rowEntries( table,table->currentRow() ) ;
+	int s = table->currentRow() ;
+
+	if( s == -1 ){
+
+		return ;
+	}
+
+	auto cp = tablewidget::rowEntries( table,s ) ;
 
 	if( cp.size() > 0 ){
 
-		favorites2::instance( this,favorites::type::others,[ this ](){
+		favorites2::instance( this,m_secrets,[ this ](){
 
 			this->updateFavoritesInContextMenu() ;
 
-		},cp.first() ) ;
+		},cp.at( 2 ),cp.at( 0 ) ) ;
 	}
 }
 
@@ -1351,10 +1350,28 @@ void sirikali::dragEnterEvent( QDragEnterEvent * e )
 	e->accept() ;
 }
 
-void sirikali::dropEvent( QDropEvent * e )
+template< typename Function >
+static favorites::volumeList _dropEvent( QDropEvent * e,Function function )
 {
 	favorites::volumeList s ;
 
+	auto _favorite_entry = [ & ]( const QString& path )->std::pair< favorites::entry,QByteArray >{
+
+		for( const auto& it : _readFavorites() ){
+
+			if( it.first.volumePath == path ){
+
+				return { it.first,function( path ) } ;
+			}
+		}
+
+		return { path,function( path ) } ;
+	} ;
+
+	QFileInfo fileInfo ;
+
+	const auto& ff = favorites::instance() ;
+
 	for( const auto& it : e->mimeData()->urls() ){
 
 		auto m = it.path() ;
@@ -1372,25 +1389,56 @@ void sirikali::dropEvent( QDropEvent * e )
 			}
 		}
 
-		s.emplace_back( std::move( m ),QByteArray() ) ;
+		fileInfo.setFile( m ) ;
+
+		if( fileInfo.isDir() ){
+
+			s.emplace_back( _favorite_entry( m ) ) ;
+
+		}else if( fileInfo.isFile() ){
+
+			if( m.endsWith( ".json" ) ){
+
+				auto ss = ff.readFavoriteByPath( m ) ;
+
+				if( ss ){
+
+					s.emplace_back( ss.RValue(),QByteArray() ) ;
+				}
+			}else{
+				s.emplace_back( m,QByteArray() ) ;
+			}
+		}
 	}
 
-	this->mountMultipleVolumes( std::move( s ) ) ;
+	return s ;
 }
 
-void sirikali::mount( const volumeInfo& entry,const QString& exe,const QByteArray& key )
+void sirikali::dropEvent( QDropEvent * e )
 {
-	this->disableAll() ;
+	auto m = settings::instance().autoMountBackEnd() ;
 
-	keyDialog::instance( this,
-			     m_secrets,
-			     entry,
-			     [ this ](){ this->enableAll() ; },
-			     [ this ](){ this->updateList() ; },
-			     m_autoOpenFolderOnMount,
-			     m_folderOpener,
-			     exe,
-			     key ) ;
+	if( m.isValid() ){
+
+		auto wallet = m_secrets.walletBk( m.bk() ) ;
+
+		if( wallet.open() ){
+
+			this->mountMultipleVolumes( _dropEvent( e,[ & ]( const QString& e ){
+
+				return wallet->readValue( e ) ;
+			} ) ) ;
+
+			return ;
+		}
+	}
+
+	this->mountMultipleVolumes( _dropEvent( e,[]( const QString& e ){
+
+		Q_UNUSED( e )
+
+		return QByteArray() ;
+	} ) ) ;
 }
 
 void sirikali::createVolume( QAction * ac )
@@ -1401,46 +1449,29 @@ void sirikali::createVolume( QAction * ac )
 
 		if( s == "Sshfs" ){
 
-			favorites2::instance( this,favorites::type::sshfs,[ this ](){
+			favorites2::instance( this,m_secrets,[ this ](){
 
 				this->updateFavoritesInContextMenu() ;
-			} ) ;
+
+			},s ) ;
 		}else{
-			this->mount( volumeInfo(),s ) ;
+			keyDialog::instance( this,
+					     m_secrets,
+					     volumeInfo(),
+					     [ this ](){ this->enableAll() ; },
+					     [ this ](){ this->updateList() ; },
+					     m_autoOpenFolderOnMount,
+					     m_folderOpener,
+					     s ) ;
 		}
 	}
 }
 
-void sirikali::slotMount()
-{
-	auto table = m_ui->tableWidget ;
-
-	if( table->rowCount() > 0 ){
-
-		auto row = table->currentRow() ;
-
-		this->mount( tablewidget::rowEntries( table,row ) ) ;
-	}
-}
-
-void sirikali::showMoungDialog( const volumeInfo& v )
-{
-	if( v.isNotValid() ){
-
-		DialogMsg( this ).ShowUIOK( tr( "ERROR" ),
-					    tr( "Permission To Access The Volume Was Denied\nOr\nThe Volume Is Not Supported" ) ) ;
-
-		this->enableAll() ;
-	}else{
-		this->mount( v ) ;
-	}
-}
-
-void sirikali::showMoungDialog( const QString& volume,const QString& m_point )
+void sirikali::autoMount( const QString& volume )
 {
 	if( !volume.isEmpty() ){
 
-		this->mount( { volume,m_point } ) ;
+		this->mountMultipleVolumes( { { volume,{} } } ) ;
 	}
 }
 
@@ -1463,7 +1494,7 @@ void sirikali::unlockVolume( bool dir )
 
 		this->enableAll() ;
 	}else{
-		this->showMoungDialog( path ) ;
+		this->autoMount( path ) ;
 	}
 }
 
@@ -1619,7 +1650,19 @@ void sirikali::runIntervalCustomCommand( const QString& cmd )
 		auto b = utility::Task::makePath( s.mountPoint ) ;
 		auto c = s.volumeType ;
 
-		auto key = utility::getKey( s.cipherPath,m_secrets ) ;
+		QString key ;
+
+		auto& settings = settings::instance() ;
+
+		if( settings.allowExternalToolsToReadPasswords() ){
+
+			auto bk = settings.autoMountBackEnd() ;
+
+			if( bk.isValid() ){
+
+				key = m_secrets.walletBk( bk.bk() ).getKey( s.cipherPath ).key ;
+			}
+		}
 
 		Task::exec( [ = ](){
 
