@@ -72,91 +72,18 @@ class QEvent ;
 
 namespace utility
 {
-	class raii
-	{
-	public:
-		raii( std::function< void() > s ) : m_function( std::move( s ) )
-		{
-		}
-		~raii()
-		{
-			if( m_run ){
+	using qstringlist_result = utility2::result< QStringList > ;
+	using qbytearray_result  = utility2::result< QByteArray > ;
+	using qstring_result     = utility2::result< QString > ;
+	using bool_result        = utility2::result< bool > ;
+	using int_result         = utility2::result< int > ;
 
-				m_function() ;
-			}
-		}
-		void cancel()
-		{
-			m_run = false ;
-		}
-	private:
-		bool m_run = true ;
-		std::function< void() > m_function ;
-	};
-	template< typename T >
-	class result
-	{
-	public:
-		result()
-		{
-		}
-		result( T e ) : m_valid( true ),m_value( std::move( e ) )
-		{
-		}
-		T * operator->()
-		{
-			return &m_value ;
-		}
-		const T * operator->() const
-		{
-			return &m_value ;
-		}
-		T& operator*()
-		{
-			return m_value ;
-		}
-		const T& operator*() const
-		{
-			return m_value ;
-		}
-		operator bool()
-		{
-			return m_valid ;
-		}
-		bool has_value() const
-		{
-			return m_valid ;
-		}
-		T& value()
-		{
-			return m_value ;
-		}
-		const T& value() const
-		{
-			return m_value ;
-		}
-		void set( T value )
-		{
-			m_value = std::move( value ) ;
-			m_valid = true ;
-		}
-	private:
-		bool m_valid = false ;
-		T m_value ;
-	} ;	
-
-	using qbytearray_result = result< QString > ;
-	using qstring_result    = result< QString > ;
-	using bool_result       = result< bool > ;
-	using int_result        = result< int > ;
-}
-
-namespace utility
-{
 	struct debug
 	{
 		static void showDebugWindow( const QString& ) ;
 
+		static void logErrorWhileStarting( const QString& ) ;
+
 		struct cout{
 
 			template< typename T >
@@ -228,13 +155,6 @@ namespace utility
 		utility::debug operator<<( const QStringList& ) ;
 	};
 
-	struct wallet
-	{
-		bool opened ;
-		bool notConfigured ;
-		QString key ;
-	};
-
 	struct fsInfo
 	{
 		bool valid ;
@@ -257,10 +177,6 @@ namespace utility
 
 	bool printVersionOrHelpInfo( const QStringList& ) ;
 
-	QString getKey( const QString& keyID,const secrets& secret ) ;
-
-	wallet getKey( const QString& keyID,LXQt::Wallet::Wallet&,QWidget * = nullptr ) ;
-
 	QString cmdArgumentValue( const QStringList&,const QString& arg,const QString& defaulT = QString() ) ;
 
 	QByteArray readPassword( bool = true ) ;
@@ -307,7 +223,7 @@ namespace utility
 
 	void wait( int ) ;
 
-	void waitForFinished( QProcess& ) ;
+	bool waitForFinished( QProcess&,int timeOut = 5 ) ;
 
 	void setMainQWidget( QWidget * ) ;
 	QWidget * mainQWidget() ;
@@ -346,7 +262,7 @@ namespace utility
 
 	void setDefaultMountPointPrefix( const QString& path ) ;
 
-	utility::result< QByteArray > yubiKey( const QByteArray& challenge ) ;
+	qbytearray_result yubiKey( const QByteArray& challenge ) ;
 
 	QString mountPathPostFix( const QString& path ) ;
 	QString mountPathPostFix( const QString& prefix,const QString& path ) ;
@@ -393,6 +309,8 @@ namespace utility
 	bool eventFilter( QObject * gui,QObject * watched,QEvent * event,std::function< void() > ) ;
 	void licenseInfo( QWidget * ) ;
 
+	void applicationStarted() ;
+
 	QString removeOption( const QStringList&,const QString& option ) ;
 	QString removeOption( const QString& commaSeparatedString,const QString& option ) ;
 
@@ -409,8 +327,6 @@ namespace utility
 
 	SocketPaths socketPath() ;
 
-	::Task::future< utility::result< QString > >& backEndInstalledVersion( const QString& backend ) ;
-
 	::Task::future< bool >& openPath( const QString& path,const QString& opener ) ;
 
 	void openPath( const QString& path,const QString& opener,QWidget *,const QString&,const QString& ) ;
@@ -644,7 +560,7 @@ namespace utility
 		bool m_finished = false ;
 	};
 
-	using task_result = utility::result< utility::Task > ;
+	using task_result = utility2::result< utility::Task > ;
 }
 
 #endif // MISCFUNCTIONS_H
